if (not Grid) then return end

-- Libraries

local L = AceLibrary("AceLocale-2.2"):new("GridLayoutPlus")
local RL = AceLibrary("Roster-2.1")

-- New Grid module

GridLayoutPlus = Grid:NewModule("GridLayoutPlus", "AceConsole-2.0")

GridLayoutPlus.defaultDB = {
	autoLayout = true,
	layoutSorting = "By Group",
	withPets = true,
	withTanks = false,
	maxColumns = 17,
	warnLayoutChanged = false,
	warnPlayersHidden = true,
	warnPetsHidden = false,
	warnTanksHidden = false,
}


-- Locals

local hasBlizzTanks = nil
local hasRATanks = nil

local ORDER_PLUS = 40

local classOrder = "WARRIOR,ROGUE,DRUID,PALADIN,SHAMAN,PRIEST,MAGE,WARLOCK,HUNTER"
local petOrder = "HUNTER,WARLOCK"
local roleOrder = "MAINTANK,MAINASSIST"


-- Init

function GridLayoutPlus:OnEnable()
	local options = GridLayout.options.args
	options["PlusHeader"] = {
		type = "header",
		order = ORDER_PLUS,
	}
	ORDER_PLUS = ORDER_PLUS + 1
	options["autoLayout"] = {
		type = "toggle",
		name = L["Auto adjust layout (+)"],
		desc = L["Automatically adjust layout to fit raid size"],
		order = ORDER_PLUS,
		get = function ()
			return GridLayoutPlus.db.profile.autoLayout
		end,
		set = function (v)
			GridLayoutPlus.db.profile.autoLayout = v
			GridLayoutPlus:UpdateLayout("FullUpdate")
		end,
	}
	ORDER_PLUS = ORDER_PLUS + 1
	options["layoutSorting"] = {
		type = "text",
		name = L["Layout sorting (+)"],
		desc = L["Choose between By Group or By Class"],
		order = ORDER_PLUS + 1,
		get = function()
			return GridLayoutPlus.db.profile.layoutSorting
		end,
		set = function(v)
			GridLayoutPlus.db.profile.layoutSorting = v
			GridLayoutPlus:UpdateLayout("FullUpdate")
		end,
		validate = {
			["By Group"] = L["By Group"],
			["By Class"] = L["By Class"],
		},
	}
	ORDER_PLUS = ORDER_PLUS + 1
	options["withPets"] = {
		type = "toggle",
		name = L["Use pet layouts (+)"],
		desc = L["Use pet layouts when auto adjusting (if possible)"],
		order = ORDER_PLUS,
		get = function ()
			return GridLayoutPlus.db.profile.withPets
		end,
		set = function (v)
			GridLayoutPlus.db.profile.withPets = v
			GridLayoutPlus:UpdateLayout("RosterUpdate")
		end,
	}
	ORDER_PLUS = ORDER_PLUS + 1
	options["withTanks"] = {
		type = "toggle",
		name = L["Use tank layouts (+)"],
		desc = L["Use tank layouts when auto adjusting (if possible)"],
		order = ORDER_PLUS,
		get = function ()
			return GridLayoutPlus.db.profile.withTanks
		end,
		set = function (v)
			GridLayoutPlus.db.profile.withTanks = v
			GridLayoutPlus:UpdateLayout("FullUpdate")
		end,
	}
	ORDER_PLUS = ORDER_PLUS + 1
	options["PlusAdvanced"] = {
		type = "group",
		name = L["Plus Advanced (+)"],
		desc = L["Advanced options for GridLayoutPlus"],
		order = ORDER_PLUS,
		args = {
			["maxColumns"] = {
				type = "range",
				name = L["Max number of columns"],
				desc = L["Maximum number of columns when auto-adjusting (WARNING: May hide units)"],
				order = 1,
				min = 1,
				max = 17,
				step = 1,
				get = function ()
					return GridLayoutPlus.db.profile.maxColumns
				end,
				set = function (v)
					GridLayoutPlus.db.profile.maxColumns = v
					GridLayoutPlus:UpdateLayout("RosterUpdate")
				end,
			},
			["warnLayoutChanged"] = {
				type = "toggle",
				name = L["Warn when layout changes"],
				desc = L["Show warning when the layout is automatically changed"],
				order = 2,
				get = function ()
					return GridLayoutPlus.db.profile.warnLayoutChanged
				end,
				set = function (v)
					GridLayoutPlus.db.profile.warnLayoutChanged = v
				end,
			},
			["warnPlayersHidden"] = {
				type = "toggle",
				name = L["Warn if players are hidden"],
				desc = L["Show warning when player units are forced to be hidden"],
				order = 3,
				get = function ()
					return GridLayoutPlus.db.profile.warnPlayersHidden
				end,
				set = function (v)
					GridLayoutPlus.db.profile.warnPlayersHidden = v
					GridLayoutPlus:UpdateLayout("FullUpdate")
				end,
			},
			["warnPetsHidden"] = {
				type = "toggle",
				name = L["Warn if pets are hidden"],
				desc = L["Show warning when pet units are forced to be hidden"],
				order = 4,
				get = function ()
					return GridLayoutPlus.db.profile.warnPetsHidden
				end,
				set = function (v)
					GridLayoutPlus.db.profile.warnPetsHidden = v
					GridLayoutPlus:UpdateLayout("FullUpdate")
				end,
			},
			["warnTanksHidden"] = {
				type = "toggle",
				name = L["Warn if tanks are hidden"],
				desc = L["Show warning when tank units are forced to be hidden"],
				order = 5,
				get = function ()
					return GridLayoutPlus.db.profile.warnTanksHidden
				end,
				set = function (v)
					GridLayoutPlus.db.profile.warnTanksHidden = v
					GridLayoutPlus:UpdateLayout("FullUpdate")
				end,
			},
		},
	}
	self:RegisterEvent("RosterLib_RosterUpdated", "RosterUpdated")
	self:RegisterEvent("RosterLib_UnitChanged", "UnitChanged")
	if oRA then
		self:RegisterEvent("oRA_MainTankUpdate", "RATanksUpdate")
		self:RegisterEvent("oRA_JoinedRaid", "RATanksUpdate")
	elseif CT_RA_MainTanks then
		hooksecurefunc("CT_RAOptions_UpdateMTs", function() self:RATankUpdate() end)
	end
	if (not InCombatLockdown()) then
		self:UpdateLayout("FullUpdate")
	else
		self:ScheduleLeaveCombatAction(self, "LeaveCombat")
	end
end


-- Additional Layouts

GridLayout:AddLayout(L["By Group 40 (+)"], {
	[1] = {
		groupFilter = "1",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[2] = {
		groupFilter = "2",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[3] = {
		groupFilter = "3",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[4] = {
		groupFilter = "4",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[5] = {
		groupFilter = "5",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[6] = {
		groupFilter = "6",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[7] = {
		groupFilter = "7",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[8] = {
		groupFilter = "8",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
})

GridLayout:AddLayout(L["By Group 40 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[2] = {
		groupFilter = "2",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[3] = {
		groupFilter = "3",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[4] = {
		groupFilter = "4",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[5] = {
		groupFilter = "5",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[6] = {
		groupFilter = "6",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[7] = {
		groupFilter = "7",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[8] = {
		groupFilter = "8",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[9] = {
		isPetGroup = true,
		groupFilter = petOrder,
		groupBy = "CLASS",
		groupingOrder = petOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 25 (+)"], {
	[1] = {
		groupFilter = "1",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[2] = {
		groupFilter = "2",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[3] = {
		groupFilter = "3",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[4] = {
		groupFilter = "4",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[5] = {
		groupFilter = "5",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
})

GridLayout:AddLayout(L["By Group 25 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[2] = {
		groupFilter = "2",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[3] = {
		groupFilter = "3",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[4] = {
		groupFilter = "4",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[5] = {
		groupFilter = "5",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[6] = {
		isPetGroup = true,
		groupFilter = petOrder,
		groupBy = "CLASS",
		groupingOrder = petOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 15 (+)"], {
	[1] = {
		groupFilter = "1",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[2] = {
		groupFilter = "2",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[3] = {
		groupFilter = "3",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
})

GridLayout:AddLayout(L["By Group 15 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[2] = {
		groupFilter = "2",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[3] = {
		groupFilter = "3",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[4] = {
		isPetGroup = true,
		groupFilter = petOrder,
		groupBy = "CLASS",
		groupingOrder = petOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 10 (+)"], {
	[1] = {
		groupFilter = "1",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[2] = {
		groupFilter = "2",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
})

GridLayout:AddLayout(L["By Group 10 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[2] = {
		groupFilter = "2",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[3] = {
		isPetGroup = true,
		groupFilter = petOrder,
		groupBy = "CLASS",
		groupingOrder = petOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 5 (+)"], {
	[1] = {
		groupFilter = "1",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
})

GridLayout:AddLayout(L["By Group 5 w/Pets (+)"], {
	[1] = {
		groupFilter = "1",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[2] = {
		isPetGroup = true,
		groupFilter = petOrder,
		groupBy = "CLASS",
		groupingOrder = petOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Sorted Class (+)"], {
	[1] = {
		groupFilter = classOrder,
		groupBy = "CLASS",
		groupingOrder = classOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
	},
})

GridLayout:AddLayout(L["By Sorted Class w/Pets (+)"], {
	[1] = {
		groupFilter = classOrder,
		groupBy = "CLASS",
		groupingOrder = classOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
	},
	[2] = {
		isPetGroup = true,
		groupFilter = petOrder,
		groupBy = "CLASS",
		groupingOrder = petOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 25 w/BlizzTanks (+)"], {
	[1] = {
		groupFilter = roleOrder,
		groupingOrder = roleOrder,
		groupBy = "ROLE"
	},
	[2] = {
		groupFilter = "1",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[3] = {
		groupFilter = "2",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[4] = {
		groupFilter = "3",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[5] = {
		groupFilter = "4",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[6] = {
		groupFilter = "5",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
})

GridLayout:AddLayout(L["By Group 25 w/BlizzTanks w/Pets (+)"], {
	[1] = {
		groupFilter = roleOrder,
		groupingOrder = roleOrder,
		groupBy = "ROLE"
	},
	[2] = {
		groupFilter = "1",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[3] = {
		groupFilter = "2",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[4] = {
		groupFilter = "3",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[5] = {
		groupFilter = "4",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[6] = {
		groupFilter = "5",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[7] = {
		isPetGroup = true,
		groupFilter = petOrder,
		groupBy = "CLASS",
		groupingOrder = petOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Group 10 w/BlizzTanks (+)"], {
	[1] = {
		groupFilter = roleOrder,
		groupingOrder = roleOrder,
		groupBy = "ROLE"
	},
	[2] = {
		groupFilter = "1",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[3] = {
		groupFilter = "2",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
})

GridLayout:AddLayout(L["By Group 10 w/BlizzTanks w/Pets (+)"], {
	[1] = {
		groupFilter = roleOrder,
		groupingOrder = roleOrder,
		groupBy = "ROLE"
	},
	[2] = {
		groupFilter = "1",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[3] = {
		groupFilter = "2",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[4] = {
		isPetGroup = true,
		groupFilter = petOrder,
		groupBy = "CLASS",
		groupingOrder = petOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayout:AddLayout(L["By Sorted Class w/BlizzTanks (+)"], {
	[1] = {
		groupFilter = roleOrder,
		groupingOrder = roleOrder,
		groupBy = "ROLE"
	},
	[2] = {
		groupFilter = classOrder,
		groupBy = "CLASS",
		groupingOrder = classOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
	},
})

GridLayout:AddLayout(L["By Sorted Class w/BlizzTanks w/Pets (+)"], {
	[1] = {
		groupFilter = roleOrder,
		groupingOrder = roleOrder,
		groupBy = "ROLE"
	},
	[2] = {
		groupFilter = classOrder,
		groupBy = "CLASS",
		groupingOrder = classOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
	},
	[3] = {
		isPetGroup = true,
		groupFilter = petOrder,
		groupBy = "CLASS",
		groupingOrder = petOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
})

GridLayoutPlus.RATanks25 = {
	[1] = {
		nameList = "",
		sortMethod = "INDEX",
	},
	[2] = {
		groupFilter = "1",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[3] = {
		groupFilter = "2",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[4] = {
		groupFilter = "3",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[5] = {
		groupFilter = "4",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
	[6] = {
		groupFilter = "5",
		groupBy = "CLASS",
		groupingOrder = classOrder,
	},
}

GridLayoutPlus.RATanks25Pets = {
	[1] = {
		nameList = "",
		sortMethod = "INDEX",
	},
	[2] = {
		groupFilter = "1",
	},
	[3] = {
		groupFilter = "2",
	},
	[4] = {
		groupFilter = "3",
	},
	[5] = {
		groupFilter = "4",
	},
	[6] = {
		groupFilter = "5",
	},
	[7] = {
		isPetGroup = true,
		groupFilter = petOrder,
		groupBy = "CLASS",
		groupingOrder = petOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
}

GridLayoutPlus.RATanks10 = {
	[1] = {
		nameList = "",
		sortMethod = "INDEX",
	},
	[2] = {
		groupFilter = "1",
	},
	[3] = {
		groupFilter = "2",
	},
}

GridLayoutPlus.RATanks10Pets = {
	[1] = {
		nameList = "",
		sortMethod = "INDEX",
	},
	[2] = {
		groupFilter = "1",
	},
	[3] = {
		groupFilter = "2",
	},
	[4] = {
		isPetGroup = true,
		groupFilter = petOrder,
		groupBy = "CLASS",
		groupingOrder = petOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
}

GridLayoutPlus.RATanksClass = {
	[1] = {
		nameList = "",
		sortMethod = "INDEX",
	},
	[2] = {
		groupFilter = classOrder,
		groupBy = "CLASS",
		groupingOrder = classOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
	},
}

GridLayoutPlus.RATanksClassPets = {
	[1] = {
		nameList = "",
		sortMethod = "INDEX",
	},
	[2] = {
		groupFilter = classOrder,
		groupBy = "CLASS",
		groupingOrder = classOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
	},
	[3] = {
		isPetGroup = true,
		groupFilter = petOrder,
		groupBy = "CLASS",
		groupingOrder = petOrder,
		unitsPerColumn = 5,
		maxColumns = 8,
		filterOnPet = true,
	},
}


GridLayout:AddLayout(L["By Group 25 w/RATanks (+)"], GridLayoutPlus.RATanks25)
GridLayout:AddLayout(L["By Group 25 w/RATanks w/Pets (+)"], GridLayoutPlus.RATanks25Pets)
GridLayout:AddLayout(L["By Group 10 w/RATanks (+)"], GridLayoutPlus.RATanks10)
GridLayout:AddLayout(L["By Group 10 w/RATanks w/Pets (+)"], GridLayoutPlus.RATanks10Pets)
GridLayout:AddLayout(L["By Sorted Class w/RATanks (+)"], GridLayoutPlus.RATanks10)
GridLayout:AddLayout(L["By Sorted Class w/RATanks w/Pets (+)"], GridLayoutPlus.RATanks10Pets)


-- Main function

function GridLayoutPlus:UpdateLayout(e)
	if GridLayoutPlus.db.profile.autoLayout and (GetNumRaidMembers() > 0) and InCombatLockdown() then
		self:ScheduleLeaveCombatAction(self, "LeaveCombat")
		return
	elseif GridLayoutPlus.db.profile.autoLayout and (GetNumRaidMembers() > 0) and (not InCombatLockdown()) then
		local maxsubgroup = 0
		local newlayout = nil
		local newmaxraidmembers = 40
		local newwithpets = true
		local newwithtanks = true
		local numberofpotentialpets = 0
		local ratanks = nil
		local blizztanks = nil
		local ratanktable = nil
		local ratanklist = nil
		if (e == "RosterUpdate") or (e == "FullUpdate") or hasBlizzTanks then
			for u in RL:IterateRoster(nil) do
				if u.subgroup > maxsubgroup then
					maxsubgroup = u.subgroup
				end
				if (u.class == "HUNTER") or (u.class == "WARLOCK") then
					numberofpotentialpets = numberofpotentialpets + 1
				end
				if (u.role == "MAINTANK") or (u.role == "MAINASSIST") then
					blizztanks = true
				end
			end
			hasBlizzTanks = blizztanks
		end
		if oRA and oRA.maintanktable and ((e == "RATankUpdate") or (e == "FullUpdate") or hasRATanks) then
			ratanktable = oRA.maintanktable
		elseif CT_RA_MainTanks and ((e == "RATankUpdate") or (e == "FullUpdate") or hasRATanks) then
			ratanktable = CT_RA_MainTanks
		end
		if ratanktable then
			for key, val in pairs(ratanktable) do
				local unitid = RL:GetUnitIDFromName(val)
				if unitid and UnitExists(unitid) and UnitPlayerOrPetInRaid(unitid) then
					ratanks = true
				end
			end
			hasRATanks = ratanks
		end
		if ratanks then
			if oRA and oRA.maintanktable then
				ratanklist = table.concat(oRA.maintanktable, ',')
			elseif CT_RA_MainTanks then
				ratanklist = table.concat(CT_RA_MainTanks, ',')
			end
			GridLayoutPlus.RATanks25[1].nameList = ratanklist
			GridLayoutPlus.RATanks25Pets[1].nameList = ratanklist
			GridLayoutPlus.RATanks10[1].nameList = ratanklist
			GridLayoutPlus.RATanks10Pets[1].nameList = ratanklist
			GridLayoutPlus.RATanksClass[1].nameList = ratanklist
			GridLayoutPlus.RATanksClassPets[1].nameList = ratanklist
		end
		if (GridLayoutPlus.db.profile.layoutSorting == "By Group") then
			if (maxsubgroup > 5) and GridLayoutPlus.db.profile.withPets and (numberofpotentialpets > 0) and (GridLayoutPlus.db.profile.maxColumns > (maxsubgroup + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 40 w/Pets (+)"]
				newwithtanks = nil
			elseif (maxsubgroup > 5) and (GridLayoutPlus.db.profile.maxColumns > 7) then
				newlayout = L["By Group 40 (+)"]
				newwithpets = nil
				newwithtanks = nil
			elseif (maxsubgroup > 2) and GridLayoutPlus.db.profile.withTanks and ratanks and GridLayoutPlus.db.profile.withPets and (numberofpotentialpets > 0) and (GridLayoutPlus.db.profile.maxColumns > (maxsubgroup+1 + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 25 w/RATanks w/Pets (+)"]
				newmaxraidmembers = 25
			elseif (maxsubgroup > 2) and GridLayoutPlus.db.profile.withTanks and blizztanks and GridLayoutPlus.db.profile.withPets and (numberofpotentialpets > 0) and (GridLayoutPlus.db.profile.maxColumns > (maxsubgroup+1 + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 25 w/BlizzTanks w/Pets (+)"]
				newmaxraidmembers = 25
			elseif (maxsubgroup > 3) and GridLayoutPlus.db.profile.withPets and (numberofpotentialpets > 0) and (GridLayoutPlus.db.profile.maxColumns > 5) then
				newlayout = L["By Group 25 w/Pets (+)"]
				newmaxraidmembers = 25
				newwithtanks = nil
			elseif (maxsubgroup > 2) and GridLayoutPlus.db.profile.withTanks and ratanks and (GridLayoutPlus.db.profile.maxColumns > 5) then
				newlayout = L["By Group 25 w/RATanks (+)"]
				newmaxraidmembers = 25
				newwithpets = nil
			elseif (maxsubgroup > 2) and GridLayoutPlus.db.profile.withTanks and blizztanks and (GridLayoutPlus.db.profile.maxColumns > 5) then
				newlayout = L["By Group 25 w/BlizzTanks (+)"]
				newmaxraidmembers = 25
				newwithpets = nil
			elseif (maxsubgroup > 3) and (GridLayoutPlus.db.profile.maxColumns > 4) then
				newlayout = L["By Group 25 (+)"]
				newmaxraidmembers = 25
				newwithpets = nil
				newwithtanks = nil
			elseif (maxsubgroup > 2) and GridLayoutPlus.db.profile.withPets and (numberofpotentialpets > 0) and (GridLayoutPlus.db.profile.maxColumns > (maxsubgroup + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 15 w/Pets (+)"]
				newmaxraidmembers = 15
				newwithtanks = nil
			elseif (maxsubgroup > 2) and (GridLayoutPlus.db.profile.maxColumns > 2) then
				newlayout = L["By Group 15 (+)"]
				newmaxraidmembers = 15
				newwithpets = nil
				newwithtanks = nil
			elseif GridLayoutPlus.db.profile.withTanks and ratanks and GridLayoutPlus.db.profile.withPets and (numberofpotentialpets > 0) and (GridLayoutPlus.db.profile.maxColumns > (maxsubgroup+1 + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 10 w/RATanks w/Pets (+)"]
				newmaxraidmembers = 10
			elseif GridLayoutPlus.db.profile.withTanks and blizztanks and GridLayoutPlus.db.profile.withPets and (numberofpotentialpets > 0) and (GridLayoutPlus.db.profile.maxColumns > (maxsubgroup+1 + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 10 w/BlizzTanks w/Pets (+)"]
				newmaxraidmembers = 10
			elseif (maxsubgroup > 1) and GridLayoutPlus.db.profile.withPets and (numberofpotentialpets > 0) and (GridLayoutPlus.db.profile.maxColumns > (maxsubgroup + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 10 w/Pets (+)"]
				newmaxraidmembers = 10
				newwithtanks = nil
			elseif GridLayoutPlus.db.profile.withTanks and ratanks and (GridLayoutPlus.db.profile.maxColumns > 2) then
				newlayout = L["By Group 10 w/RATanks (+)"]
				newmaxraidmembers = 10
				newwithpets = nil
			elseif GridLayoutPlus.db.profile.withTanks and blizztanks and (GridLayoutPlus.db.profile.maxColumns > 2) then
				newlayout = L["By Group 10 w/BlizzTanks (+)"]
				newmaxraidmembers = 10
				newwithpets = nil
			elseif (maxsubgroup > 1) and (GridLayoutPlus.db.profile.maxColumns > 1) then
				newlayout = L["By Group 10 (+)"]
				newmaxraidmembers = 10
				newwithpets = nil
				newwithtanks = nil
			elseif GridLayoutPlus.db.profile.withPets and (numberofpotentialpets > 0) and (GridLayoutPlus.db.profile.maxColumns > (maxsubgroup + (numberofpotentialpets / 5))) then
				newlayout = L["By Group 5 w/Pets (+)"]
				newmaxraidmembers = 5
				newwithtanks = nil
			else
				newlayout = L["By Group 5 (+)"]
				newmaxraidmembers = 5
				newwithpets = nil
				newwithtanks = nil
			end
		elseif (GridLayoutPlus.db.profile.layoutSorting == "By Class") then
			if GridLayoutPlus.db.profile.withTanks and ratanks and GridLayoutPlus.db.profile.withPets and (GridLayoutPlus.db.profile.maxColumns > ((GetNumRaidMembers() / 5) + 1 + (numberofpotentialpets / 5))) then
				newlayout = L["By Sorted Class w/RATanks w/Pets (+)"]
			elseif GridLayoutPlus.db.profile.withTanks and blizztanks and GridLayoutPlus.db.profile.withPets and (GridLayoutPlus.db.profile.maxColumns > ((GetNumRaidMembers() / 5) + 1 + (numberofpotentialpets / 5))) then
				newlayout = L["By Sorted Class w/BlizzTanks w/Pets (+)"]
			elseif GridLayoutPlus.db.profile.withPets and (GridLayoutPlus.db.profile.maxColumns > ((GetNumRaidMembers() / 5) + (numberofpotentialpets / 5))) then
				newlayout = L["By Sorted Class w/Pets (+)"]
				newwithtanks = nil
			elseif GridLayoutPlus.db.profile.withTanks and ratanks and (GridLayoutPlus.db.profile.maxColumns > ((GetNumRaidMembers() / 5) + 1)) then
				newlayout = L["By Sorted Class w/RATanks (+)"]
				newwithpets = nil
			elseif GridLayoutPlus.db.profile.withTanks and blizztanks and (GridLayoutPlus.db.profile.maxColumns > ((GetNumRaidMembers() / 5) + 1)) then
				newlayout = L["By Sorted Class w/BlizzTanks (+)"]
				newwithpets = nil
			else
				newlayout = L["By Sorted Class (+)"]
				newwithpets = nil
				newwithtanks = nil
			end
		end
		if newlayout and (GridLayoutPlus.db.profile.layout ~= newlayout) then
			GridLayoutPlus.db.profile.layout = newlayout
			GridLayout:LoadLayout(newlayout)
			if GridLayoutPlus.db.profile.warnLayoutChanged then
				self:Print(L["Your layout has automatically changed"])
			end
			if GridLayoutPlus.db.profile.warnPlayersHidden and (GetNumRaidMembers() > newmaxraidmembers) then
				self:Print(L["Your settings are causing player units to be hidden"])
			end
			if GridLayoutPlus.db.profile.warnPetsHidden and GridLayoutPlus.db.profile.withPets and (not newwithpets) and (numberofpotentialpets > 0) then
				self:Print(L["Your settings are causing pet units to be hidden"])
			end
			if GridLayoutPlus.db.profile.warnTanksHidden and GridLayoutPlus.db.profile.withTanks and (tanklist or blizzmaintank) and (not newwithtanks) then
				self:Print(L["Your settings are causing tank units to be hidden"])
			end
		end
	end
end


-- Event handlers

function GridLayoutPlus:RosterUpdated()
	self:UpdateLayout("RosterUpdate")
end

function GridLayoutPlus:UnitChanged(_, _, _, _, _, _, _, _, _, _, newrole, _, oldrole)
	if ((newrole == "MAINTANK") or (newrole == "MAINASSIST") or (oldrole == "MAINTANK") or (oldrole == "MAINASSIST")) and (newrole ~= oldrole) then
		self:UpdateLayout("RosterUpdate")
	end
end

function GridLayoutPlus:RATanksUpdate()
	self:UpdateLayout("RATankUpdate")
end

function GridLayoutPlus:LeaveCombat()
	self:UpdateLayout("FullUpdate")
end